package com.malk.service.dingtalk.impl;

import com.malk.Util.*;
import com.malk.server.common.FilePath;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_Contacts;
import com.malk.service.dingtalk.DDClient_Storage;
import com.malk.service.dingtalk.DDClient_Workflow;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class DDClientImpl implements DDClient {

    @Autowired
    private DDConf ddConf;

    /**
     * 获取访问授权
     *
     * @apiNote https://open.dingtalk.com/document/personalapp-service/obtain-user-token
     */
    @Synchronized
    @Override
    public String getAccessToken() {
        String accessToken = UtilToken.get("invalid-token-dingtalk");
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map param = new HashMap();
        param.put("appkey", ddConf.getAppKey());
        param.put("appsecret", ddConf.getAppSecret());
        DDR r = (DDR) UtilHttp.doGet("https://oapi.dingtalk.com/gettoken", param, DDR.class);
        log.info("响应token, {}", r.getAccessToken());
        accessToken = r.getAccessToken();
        // token失效自动重置: DD重新调用会重置过期时间
        UtilToken.put("invalid-token-dingtalk", accessToken, r.getExpiresIn() * 1000L);
        return accessToken;
    }

    @Override
    public String getAccessToken(String appKey, String appSecret) {
        Map param = UtilMap.map("appkey, appsecret", appKey, appSecret);
        DDR r = DDR.doGet("https://oapi.dingtalk.com/gettoken", null, param);
        log.info("响应token, {}", r.getAccessToken());
        return r.getAccessToken();
    }

    /**
     * token授权参数: 旧版本
     */
    @Override
    public Map initTokenParams() {
        return DDConf.initTokenParams(getAccessToken());
    }

    /**
     * token授权参数: 新版本
     */
    @Override
    public Map initTokenHeader() {
        return DDConf.initTokenHeader(getAccessToken());
    }

    @Autowired
    private DDClient_Storage ddClient_storage;

    @Autowired
    private DDClient_Contacts ddClient_contacts;

    @Autowired
    private FilePath filePath;

    @Autowired
    private DDClient_Workflow ddClient_workflow;

    /**
     * 上传审批附件
     *
     * @param urlFile      远程文件地址
     * @param fileNamePure 文件名称[后缀自动通过地址获取]
     */
    @Override
    public Map uploadFileFormUrl(String accessToken, String userId, String urlFile, String fileNamePure) {
        // 下载到本地
        String fileName = fileNamePure + urlFile.substring(urlFile.lastIndexOf(".")).toLowerCase();
        File file = UtilFile.mkdirIfNot(fileName, filePath.getPath().getFile());
        UtilHttp.doDownload(urlFile, file);
        return uploadFileFormUrl(accessToken, userId, file.getAbsolutePath());
    }

    @Override
    public Map uploadFileFormUrl(String accessToken, String userId, String filePath) {
        // 获取用户unionId
        String unionId = String.valueOf(ddClient_contacts.getUserInfoById(accessToken, userId).get("unionid"));
        // 获取储存空间
        String spaceId = ddClient_storage.getSpacesInfos(accessToken, userId, null);
        // 添加空间用户权限
        List<Map> members = Arrays.asList(UtilMap.map("type, id, corpId", "USER", unionId, ddConf.getCorpId()));
        ddClient_storage.addSpacePermissions(accessToken, spaceId, "0", unionId, "EDITOR", members, null);
        // 获取空间上传信息
        DDR_New ddr_new = ddClient_storage.getUploadInfos(accessToken, spaceId, unionId, null);
        // 执行文件上传
        String resourceUrl = ((List<String>) ddr_new.getHeaderSignatureInfo().get("resourceUrls")).get(0);
        ddClient_storage.uploadFiles(resourceUrl, (Map<String, String>) ddr_new.getHeaderSignatureInfo().get("headers"), filePath);
        // 提交文件
        return ddClient_storage.commitFiles(accessToken, spaceId, unionId, ddr_new.getUploadKey(), String.valueOf(UtilList.getLast(filePath.split("/"))), "0", null);
    }

    /**
     * 获取审批实例ID列表_全部
     */
    @Override
    public List<String> getInstanceIds_all(String access_token, String processCode, long startTime, long endTime, Map extInfo) {
        return _getInstanceIds_all(access_token, processCode, startTime, endTime, extInfo, new ArrayList());
    }

    // 递归查询
    private List<String> _getInstanceIds_all(String access_token, String processCode, long startTime, long endTime, Map extInfo, List dataList) {
        Map rsp = ddClient_workflow.getInstanceIds(access_token, processCode, startTime, endTime, extInfo);
        dataList.addAll(((List) rsp.get("list")));
        if (rsp.containsKey("nextToken")) {
            extInfo.put("nextToken", rsp.get("nextToken"));
            _getInstanceIds_all(access_token, processCode, startTime, endTime, extInfo, dataList);
        }
        return dataList;
    }

    /**
     * 获取全部架构内全部部门
     */
    @Override
    public List<Map> getDepartmentDetail_all(String access_token) {
        return _getDepartment_all(access_token, DDConf.TOP_DEPARTMENT, new ArrayList<>(), false);
    }

    @Override
    public List<Long> getDepartmentId_all(String access_token) {
        return (List<Long>) _getDepartment_all(access_token, DDConf.TOP_DEPARTMENT, new ArrayList<>(), true)
                .stream().map(detpId -> Long.valueOf(String.valueOf(detpId))).collect(Collectors.toList());
    }

    // 递归查询
    private List _getDepartment_all(String access_token, long deptId, List deptIdList, boolean isIdList) {
        List tList = isIdList ?
                ddClient_contacts.listSubDepartmentId(access_token, deptId) :
                ddClient_contacts.listSubDepartmentDetail(access_token, deptId);
        deptIdList.addAll(tList);
        // 当所有子部门都为空, 递归出口
        for (Object dept : tList) {
            _getDepartment_all(access_token, Long.valueOf(String.valueOf(isIdList ? dept : ((Map) dept).get("dept_id"))), deptIdList, isIdList);
        }
        return deptIdList;
    }
}